function cvc = ContrLaw(uv,z,uv_final,Fu,Fv,u0,v0,lambda)

% CONTRLAW Compute the kinematic screw for 2D image-based visual servoing.
% uv, uv_final are 2 x N matrix of pixel coordinates (current, and desired)
% z is a N x 1 vector of depth
% FU, Fv, u0, v0 are the camera intrinsic parameters
% lambda is the control gain.

n = size(z,1);

% 1ST STEP: Compute image coordinates

x = (uv(1,:)-u0)/Fu;
y = (uv(2,:)-v0)/Fv;

% 2ND STEP: Construct interaction (Jacobian) matrix

Lxy = zeros(2*n,6);
for i=1:n
Lxy(2*i-1:2*i,:) = [-1/z(i)  0   x(i)/z(i)  x(i)*y(i)  -(1+x(i)^2)   y(i);
                  0  -1/z(i)   y(i)/z(i)  1+y(i)^2   -x(i)*y(i)   -x(i)];
end

% 3RD STEP: Compute the pseudo-inverse of the interaction matrix

Lxyp = pinv(Lxy);

% 4TH STEP: Apply the proportional control law

s = zeros(2*n,1);
sd = zeros(2*n,1);

for i=1:n
    s(2*i-1:2*i) = [x(i);y(i)];
    sd(2*i-1:2*i) = [uv_final(1,i)-u0)/Fu;
                     uv_final(2,i)-v0)/Fv];
end

cvc = -lambda*Lxyp*(s-sd);
